using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using SpiderMind.Settings;

namespace SpiderMind
{
  public partial class frmMain : Form
  {
    public frmMain()
    {
      SetStyle(ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint | ControlStyles.OptimizedDoubleBuffer, true);
      InitializeComponent();
    }

    private void frmMain_Load(object sender, EventArgs e)
    {
      timerAnim.Enabled = true;
      FormWidth = this.Width;
      FormHeight = this.Height;
    }

    protected override void OnPaint(PaintEventArgs e)
    {
      Pen myPen = new Pen(Color.Red);
      Brush myBrush = new SolidBrush(Color.White);
      FontFamily myFontFamily = new FontFamily("Verdana");
      Font myFont = new Font(myFontFamily, 10, FontStyle.Regular, GraphicsUnit.Pixel);
      StringFormat myStringFormat = StringFormat.GenericDefault;
      Graphics formGraphics = e.Graphics;

      myStringFormat.Alignment = StringAlignment.Center;
      myStringFormat.LineAlignment = StringAlignment.Center;

      //formGraphics.Clear(Properties.Settings.Default.plexColorInner);
      formGraphics.SmoothingMode = SmoothingMode.AntiAlias;
      workspace_paint_background(formGraphics);
      workspace_paint_throbber(formGraphics);

      paint_Button(formGraphics, this.ClientSize.Width - 20, 5, 15);

      if (MouseIsDown)
        workspace_paint_link(formGraphics, LinkType.Local, this.Width / 2, this.Height / 2, MouseX, MouseY);

      workspace_paint_link(formGraphics, LinkType.Local, this.Width / 2, this.Height / 2, this.Width * 1 / 4, this.Height / 2 + 100);
      workspace_paint_link(formGraphics, LinkType.Local, this.Width / 2, this.Height / 2, this.Width * 2 / 4, this.Height / 2 + 100);
      workspace_paint_link(formGraphics, LinkType.Local, this.Width / 2, this.Height / 2, this.Width * 3 / 4, this.Height / 2 + 100);

      workspace_paint_node(formGraphics, NodeType.Active, false, "This is one long link!", this.Width / 2, this.Height / 2);
      workspace_paint_node(formGraphics, NodeType.Highlight, false, "Highlight Link", this.Width * 1 / 4, this.Height / 2 + 100);
      workspace_paint_node(formGraphics, NodeType.Distant, false, "Distant Link", this.Width * 2 / 4, this.Height / 2 + 100);
      workspace_paint_node(formGraphics, NodeType.Default, false, "Default Link", this.Width * 3 / 4, this.Height / 2 + 100);

      if (MouseIsDown)
        workspace_paint_node(formGraphics, NodeType.Local, false, MouseX.ToString() + ", " + MouseY.ToString(), MouseX, MouseY);

      formGraphics.DrawString(MouseX.ToString() + ", " + MouseY.ToString(), myFont, myBrush, 30f, 10f, myStringFormat);

      //formGraphics.DrawString(((DateTime.Now.Ticks / (double)10000000)).ToString(), myFont, myBrush, this.ClientRectangle.Width / 2, 20f, myStringFormat);
      //formGraphics.DrawString(Math.Truncate(LastTick).ToString(), myFont, myBrush, this.ClientRectangle.Width / 2, 30f, myStringFormat);
      //formGraphics.DrawString(((DateTime.Now.Ticks / (double)10000000) - LastTick).ToString("F4"), myFont, myBrush, this.ClientRectangle.Width / 2, 40f, myStringFormat);

      // FPS Calculation...
      FramesDrawn++;
      if (((DateTime.Now.Ticks / (double)10000000) - LastTick) >= TickDelta)
      {
        LastFPS = FPS;
        FPS = FramesDrawn / ((DateTime.Now.Ticks / (double)10000000) - LastTick);
        LastTick = (DateTime.Now.Ticks / (double)10000000);
        LastFramesDrawn = FramesDrawn;
        FramesDrawn = 0;
      }

      formGraphics.DrawString(((FPS+LastFPS)/2).ToString("F2")+" FPS", myFont, myBrush, this.ClientRectangle.Width - 40f, this.ClientRectangle.Height - 10f, myStringFormat);

      myPen.Dispose();
      myBrush.Dispose();
      myFontFamily.Dispose();
      myFont.Dispose();
      myStringFormat.Dispose();
    }

    private void timerAnim_Tick(object sender, EventArgs e)
    {
      if (!IsDrawing)
        this.Invalidate();
      RotateAngle = RotateAngle + Throbber.Default.Speed;
      if (RotateAngle > 360)
        RotateAngle = RotateAngle - 360;
    }

    private void frmMain_Resize(object sender, EventArgs e)
    {
      if ((this.Width < FormWidth) || (this.Height < FormHeight)) {
        FormWidth = this.Width;
        FormHeight = this.Height;
        this.Invalidate();
      }
    }

    private void frmMain_MouseMove(object sender, MouseEventArgs e)
    {
      MouseX = e.X;
      MouseY = e.Y;
    }

    private void frmMain_MouseEnter(object sender, EventArgs e)
    {
      MouseFocus = true;
    }

    private void frmMain_MouseLeave(object sender, EventArgs e)
    {
      MouseFocus = false;
    }

    private void frmMain_Validated(object sender, EventArgs e)
    {
      IsDrawing = false;
    }

    private void frmMain_Validating(object sender, CancelEventArgs e)
    {
      IsDrawing = true;
    }

    private void mnuPlexConfig_Click(object sender, EventArgs e)
    {
      Form cPanel = new frmConfig();
      Properties.Settings.Default.Save();
      if (cPanel.ShowDialog(this) == DialogResult.OK)
      {
        App.Default.Save();
        Node.Default.Save();
        Throbber.Default.Save();
        Workspace.Default.Save();
      }
      else
      {
        App.Default.Reload();
        Node.Default.Reload();
        Throbber.Default.Reload();
        Workspace.Default.Reload();
      }
    }

    private void frmMain_ResizeEnd(object sender, EventArgs e)
    {

    }

    private void mnuPlexExit_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
    {
      if (App.Default.ConfirmOnExit)
      {
        DialogResult myResult;
        myResult = MessageBox.Show(this, "Do you wish to exit?", "Exit?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
        if (myResult == DialogResult.No)
          e.Cancel = true;
      }
    }

    private void frmMain_MouseDown(object sender, MouseEventArgs e)
    {
      MouseIsDown = true;
    }

    private void frmMain_MouseUp(object sender, MouseEventArgs e)
    {
      MouseIsDown = false;
    }
  }
}